diff --git a/.gitignore b/.gitignore
index b1080e50..96f557f3 100644
--- a/.gitignore
+++ b/.gitignore
@@ -21,5 +21,5 @@ ev/autom4te.cache
/ipch/
/Win32/
/x64/
-
-test/runner
+test/run-tests
+test/run-benchmarks
diff --git a/liboio-benchmark.vcxproj b/liboio-benchmark.vcxproj
new file mode 100644
index 00000000..34943943
--- /dev/null
+++ b/liboio-benchmark.vcxproj
@@ -0,0 +1,157 @@
+
+
+
+
+ Debug
+ Win32
+
+
+ Debug
+ x64
+
+
+ Release
+ Win32
+
+
+ Release
+ x64
+
+
+
+ Win32Proj
+
+
+
+ Application
+ true
+
+
+ Application
+ true
+
+
+ Application
+ false
+
+
+ Application
+ false
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+ true
+ $(Platform)\$(Configuration)\
+ $(SolutionDir)$(Platform)\$(Configuration)\
+
+
+ true
+ $(Platform)\$(Configuration)\
+ $(SolutionDir)$(Platform)\$(Configuration)\
+
+
+ true
+ $(Platform)\$(Configuration)\
+ $(SolutionDir)$(Platform)\$(Configuration)\
+
+
+ true
+ $(Platform)\$(Configuration)\
+ $(SolutionDir)$(Platform)\$(Configuration)\
+
+
+
+ WIN32;_DEBUG;_CONSOLE;%(PreprocessorDefinitions)
+ MultiThreadedDebugDLL
+ Level3
+ ProgramDatabase
+ Disabled
+
+
+ MachineX86
+ true
+ Console
+ kernel32.lib;user32.lib;gdi32.lib;winspool.lib;comdlg32.lib;advapi32.lib;shell32.lib;ole32.lib;oleaut32.lib;uuid.lib;odbc32.lib;odbccp32.lib;ws2_32.lib;%(AdditionalDependencies)
+
+
+
+
+ WIN32;_DEBUG;_CONSOLE;%(PreprocessorDefinitions)
+ MultiThreadedDebugDLL
+ Level3
+ ProgramDatabase
+ Disabled
+
+
+ true
+ Console
+ kernel32.lib;user32.lib;gdi32.lib;winspool.lib;comdlg32.lib;advapi32.lib;shell32.lib;ole32.lib;oleaut32.lib;uuid.lib;odbc32.lib;odbccp32.lib;ws2_32.lib;%(AdditionalDependencies)
+
+
+
+
+ WIN32;NDEBUG;_CONSOLE;%(PreprocessorDefinitions)
+ MultiThreadedDLL
+ Level3
+ ProgramDatabase
+
+
+ MachineX86
+ true
+ Console
+ true
+ true
+ kernel32.lib;user32.lib;gdi32.lib;winspool.lib;comdlg32.lib;advapi32.lib;shell32.lib;ole32.lib;oleaut32.lib;uuid.lib;odbc32.lib;odbccp32.lib;ws2_32.lib;%(AdditionalDependencies)
+
+
+
+
+ WIN32;NDEBUG;_CONSOLE;%(PreprocessorDefinitions)
+ MultiThreadedDLL
+ Level3
+ ProgramDatabase
+
+
+ true
+ Console
+ true
+ true
+ kernel32.lib;user32.lib;gdi32.lib;winspool.lib;comdlg32.lib;advapi32.lib;shell32.lib;ole32.lib;oleaut32.lib;uuid.lib;odbc32.lib;odbccp32.lib;ws2_32.lib;%(AdditionalDependencies)
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+ {301fe650-cd34-14e5-6b63-42e383fa02bc}
+
+
+
+
+
+
\ No newline at end of file
diff --git a/liboio-test.vcxproj b/liboio-test.vcxproj
index ec48b3ee..a3315bd0 100644
--- a/liboio-test.vcxproj
+++ b/liboio-test.vcxproj
@@ -144,12 +144,13 @@
+
-
+
diff --git a/liboio.sln b/liboio.sln
index 77f39988..2c8e4aed 100644
--- a/liboio.sln
+++ b/liboio.sln
@@ -5,6 +5,10 @@ Project("{8BC9CEB8-8B4A-11D0-8D11-00A0C91BC942}") = "liboio", "liboio.vcxproj",
EndProject
Project("{8BC9CEB8-8B4A-11D0-8D11-00A0C91BC942}") = "liboio-test", "liboio-test.vcxproj", "{1D7C3F6C-A4AF-DD73-2D20-B2FC919B3744}"
EndProject
+Project("{2150E333-8FDC-42A3-9474-1A3956D46DE8}") = "Solution Items", "Solution Items", "{CB2DC484-525A-4EB1-9DCB-192C69576A3E}"
+EndProject
+Project("{8BC9CEB8-8B4A-11D0-8D11-00A0C91BC942}") = "liboio-benchmark", "liboio-benchmark.vcxproj", "{595734A6-DDB3-9EB6-906F-E8534ADD789B}"
+EndProject
Global
GlobalSection(SolutionConfigurationPlatforms) = preSolution
Debug|Win32 = Debug|Win32
@@ -29,6 +33,14 @@ Global
{1D7C3F6C-A4AF-DD73-2D20-B2FC919B3744}.Release|Win32.Build.0 = Release|Win32
{1D7C3F6C-A4AF-DD73-2D20-B2FC919B3744}.Release|x64.ActiveCfg = Release|x64
{1D7C3F6C-A4AF-DD73-2D20-B2FC919B3744}.Release|x64.Build.0 = Release|x64
+ {595734A6-DDB3-9EB6-906F-E8534ADD789B}.Debug|Win32.ActiveCfg = Debug|Win32
+ {595734A6-DDB3-9EB6-906F-E8534ADD789B}.Debug|Win32.Build.0 = Debug|Win32
+ {595734A6-DDB3-9EB6-906F-E8534ADD789B}.Debug|x64.ActiveCfg = Debug|x64
+ {595734A6-DDB3-9EB6-906F-E8534ADD789B}.Debug|x64.Build.0 = Debug|x64
+ {595734A6-DDB3-9EB6-906F-E8534ADD789B}.Release|Win32.ActiveCfg = Release|Win32
+ {595734A6-DDB3-9EB6-906F-E8534ADD789B}.Release|Win32.Build.0 = Release|Win32
+ {595734A6-DDB3-9EB6-906F-E8534ADD789B}.Release|x64.ActiveCfg = Release|x64
+ {595734A6-DDB3-9EB6-906F-E8534ADD789B}.Release|x64.Build.0 = Release|x64
EndGlobalSection
GlobalSection(SolutionProperties) = preSolution
HideSolutionNode = FALSE
diff --git a/oio-win.c b/oio-win.c
index 598756e0..b82a9060 100644
--- a/oio-win.c
+++ b/oio-win.c
@@ -19,7 +19,6 @@
* IN THE SOFTWARE.
*/
-
#include
#include
#include
diff --git a/oio-win.h b/oio-win.h
index 41e8dc91..fe9d4e67 100644
--- a/oio-win.h
+++ b/oio-win.h
@@ -19,7 +19,6 @@
* IN THE SOFTWARE.
*/
-
#ifndef _WIN32_WINNT
# define _WIN32_WINNT 0x0501
#endif
diff --git a/test/benchmark-dummy.c b/test/benchmark-dummy.c
new file mode 100644
index 00000000..f4a6b715
--- /dev/null
+++ b/test/benchmark-dummy.c
@@ -0,0 +1,28 @@
+/* Copyright Joyent, Inc. and other Node contributors. All rights reserved.
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+ * IN THE SOFTWARE.
+ */
+
+#include "task.h"
+
+
+BENCHMARK_IMPL(dummy) {
+ LOG("23487 foos/bar\n");
+ return 0;
+}
\ No newline at end of file
diff --git a/test/benchmark-list.h b/test/benchmark-list.h
new file mode 100644
index 00000000..82a08f77
--- /dev/null
+++ b/test/benchmark-list.h
@@ -0,0 +1,26 @@
+/* Copyright Joyent, Inc. and other Node contributors. All rights reserved.
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+ * IN THE SOFTWARE.
+ */
+
+BENCHMARK_DECLARE (dummy)
+
+TASK_LIST_START
+ BENCHMARK_ENTRY (dummy)
+TASK_LIST_END
\ No newline at end of file
diff --git a/test/echo-server.c b/test/echo-server.c
index d87412f4..61aa7ab5 100644
--- a/test/echo-server.c
+++ b/test/echo-server.c
@@ -20,7 +20,7 @@
*/
#include "../oio.h"
-#include "test.h"
+#include "task.h"
#include
#include
diff --git a/test/run-benchmarks.c b/test/run-benchmarks.c
new file mode 100644
index 00000000..bdf253cb
--- /dev/null
+++ b/test/run-benchmarks.c
@@ -0,0 +1,59 @@
+/* Copyright Joyent, Inc. and other Node contributors. All rights reserved.
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+ * IN THE SOFTWARE.
+ */
+
+#include
+#include
+
+#include "runner.h"
+#include "task.h"
+
+/* Actual benchmarks and helpers are defined in benchmark-list.h */
+#include "benchmark-list.h"
+
+
+/* The time in milliseconds after which a single benchmark times out. */
+#define BENCHMARK_TIMEOUT 60000
+
+
+int main(int argc, char **argv) {
+ task_entry_t *task;
+
+ platform_init();
+
+ if (argc > 1) {
+ /* A specific process was requested. */
+ run_process(argv[1]);
+
+ } else {
+ /* Run all benchmarks. */
+ task = (task_entry_t*)&TASKS;
+ for (task = (task_entry_t*)&TASKS; task->main; task++) {
+ if (task->is_helper) {
+ continue;
+ }
+
+ run_task(task, BENCHMARK_TIMEOUT, 1);
+ }
+ LOG("Done.\n");
+
+ return 0;
+ }
+}
\ No newline at end of file
diff --git a/test/run-tests.c b/test/run-tests.c
new file mode 100644
index 00000000..c46efd98
--- /dev/null
+++ b/test/run-tests.c
@@ -0,0 +1,84 @@
+/* Copyright Joyent, Inc. and other Node contributors. All rights reserved.
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
+ * IN THE SOFTWARE.
+ */
+
+#include
+#include
+
+#include "runner.h"
+#include "task.h"
+
+/* Actual tests and helpers are defined in test-list.h */
+#include "test-list.h"
+
+
+/* The time in milliseconds after which a single test times out. */
+#define TEST_TIMEOUT 20000
+
+
+static void log_progress(int total, int passed, int failed, char *name) {
+ LOGF("[%% %3d|+ %3d|- %3d]: %-50s\n", (passed + failed) / total * 100,
+ passed, failed, name);
+}
+
+
+int main(int argc, char **argv) {
+ int total, passed, failed;
+ task_entry_t *task;
+
+ platform_init();
+
+ if (argc > 1) {
+ /* A specific process was requested. */
+ run_process(argv[1]);
+
+ } else {
+ /* Count the number of tests. */
+ total = 0;
+ task = (task_entry_t*)&TASKS;
+ for (task = (task_entry_t*)&TASKS; task->main; task++) {
+ if (!task->is_helper) {
+ total++;
+ }
+ }
+
+ /* Run all tests. */
+ passed = 0;
+ failed = 0;
+ task = (task_entry_t*)&TASKS;
+ for (task = (task_entry_t*)&TASKS; task->main; task++) {
+ if (task->is_helper) {
+ continue;
+ }
+
+ log_progress(total, passed, failed, task->task_name);
+ rewind_cursor();
+
+ if (run_task(task, TEST_TIMEOUT, 0)) {
+ passed++;
+ } else {
+ failed++;
+ }
+ }
+ log_progress(total, passed, failed, "Done.");
+
+ return 0;
+ }
+}
\ No newline at end of file
diff --git a/test/runner-unix.c b/test/runner-unix.c
index cee99c61..04233966 100644
--- a/test/runner-unix.c
+++ b/test/runner-unix.c
@@ -43,6 +43,14 @@ static void get_executable_path() {
}
+/* Do platform-specific initialization. */
+void platform_init() {
+ /* Disable stdio output buffering. */
+ setvbuf(stdout, NULL, _IONBF, 0);
+ setvbuf(stderr, NULL, _IONBF, 0);
+}
+
+
/* Invoke "arv[0] test-name". Store process info in *p. */
/* Make sure that all stdio output of the processes is buffered up. */
int process_start(char* name, process_info_t* p) {
diff --git a/test/runner-win.c b/test/runner-win.c
index 0a6e6152..84726e1c 100644
--- a/test/runner-win.c
+++ b/test/runner-win.c
@@ -23,11 +23,11 @@
#include
#include
-#include "test.h"
+#include "task.h"
#include "runner.h"
-/*
+/*
* Define the stuff that MinGW doesn't have
*/
#ifndef GetFileSizeEx
@@ -36,6 +36,18 @@
#endif
+/* Do platform-specific initialization. */
+void platform_init() {
+ /* Disable the "application crashed" popup. */
+ SetErrorMode(SEM_FAILCRITICALERRORS | SEM_NOGPFAULTERRORBOX |
+ SEM_NOOPENFILEERRORBOX);
+
+ /* Disable stdio output buffering. */
+ setvbuf(stdout, NULL, _IONBF, 0);
+ setvbuf(stderr, NULL, _IONBF, 0);
+}
+
+
int process_start(char *name, process_info_t *p) {
HANDLE file = INVALID_HANDLE_VALUE;
HANDLE nul = INVALID_HANDLE_VALUE;
diff --git a/test/runner.c b/test/runner.c
index 1d5aef87..1aa4be91 100644
--- a/test/runner.c
+++ b/test/runner.c
@@ -19,32 +19,41 @@
* IN THE SOFTWARE.
*/
-#include "test.h"
-#include "runner.h"
-
-#include
#include
-/* Actual tests and helpers are defined in test-list.h */
-#include "test-list.h"
+#include "runner.h"
+#include "task.h"
-/* The maximum number of processes (main + helpers) that a test can have. */
-#define TEST_MAX_PROCESSES 8
-/* The time in milliseconds after which a single test times out, */
-#define TEST_TIMEOUT 20000
+/* Start a specific process declared by TEST_ENTRY or TEST_HELPER. */
+/* Returns the exit code of the specific process. */
+int run_process(char* name) {
+ task_entry_t *test;
+
+ for (test = (task_entry_t*)&TASKS; test->main; test++) {
+ if (strcmp(name, test->process_name) == 0) {
+ return test->main();
+ }
+ }
+
+ LOGF("Test process %s not found!\n", name);
+ return 255;
+}
/*
- * Runs an individual test; returns 1 if the test succeeded, 0 if it failed.
+ * Runs all processes associated with a particular test or benchmark.
+ * It returns 1 if the test succeeded, 0 if it failed.
* If the test fails it prints diagnostic information.
+ * If benchmark_output is nonzero, the output from the main process is
+ * always shown.
*/
-int run_test(test_entry_t *test) {
+int run_task(task_entry_t *test, int timeout, int benchmark_output) {
int i, result, success;
char errmsg[256];
- test_entry_t *helper;
+ task_entry_t *helper;
int process_count;
- process_info_t processes[TEST_MAX_PROCESSES];
+ process_info_t processes[MAX_PROCESSES];
process_info_t *main_process;
success = 0;
@@ -52,12 +61,14 @@ int run_test(test_entry_t *test) {
process_count = 0;
/* Start all helpers for this test first. */
- for (helper = (test_entry_t*)&TESTS; helper->main; helper++) {
+ for (helper = (task_entry_t*)&TASKS; helper->main; helper++) {
if (helper->is_helper &&
- strcmp(test->test_name, helper->test_name) == 0) {
+ strcmp(test->task_name, helper->task_name) == 0) {
if (process_start(helper->process_name, &processes[process_count]) == -1) {
- snprintf((char*)&errmsg, sizeof(errmsg),
- "process `%s` failed to start.", helper->process_name);
+ snprintf((char*)&errmsg,
+ sizeof(errmsg),
+ "process `%s` failed to start.",
+ helper->process_name);
goto finalize;
}
process_count++;
@@ -74,7 +85,7 @@ int run_test(test_entry_t *test) {
process_count++;
/* Wait for the main process to terminate. */
- result = process_wait(main_process, 1, TEST_TIMEOUT);
+ result = process_wait(main_process, 1, timeout);
if (result == -1) {
FATAL("process_wait failed");
} else if (result == -2) {
@@ -107,27 +118,44 @@ finalize:
/* Show error and output from processes if the test failed. */
if (!success) {
LOG("=============================================================\n");
- LOGF("Test `%s` failed: %s\n", test->test_name, errmsg);
+ LOGF("`%s` failed: %s\n", test->task_name, errmsg);
for (i = 0; i < process_count; i++) {
switch (process_output_size(&processes[i])) {
- case -1:
- LOGF("Output from process `%s`: << unavailable >>\n",
- process_get_name(&processes[i]));
- break;
+ case -1:
+ LOGF("Output from process `%s`: (unavailable)\n",
+ process_get_name(&processes[i]));
+ break;
- case 0:
- LOGF("Output from process `%s`: << no output >>\n",
- process_get_name(&processes[i]));
- break;
+ case 0:
+ LOGF("Output from process `%s`: (no output)\n",
+ process_get_name(&processes[i]));
+ break;
- default:
- LOGF("Output from process `%s`:\n", process_get_name(&processes[i]));
- process_copy_output(&processes[i], fileno(stderr));
- break;
+ default:
+ LOGF("Output from process `%s`:\n", process_get_name(&processes[i]));
+ process_copy_output(&processes[i], fileno(stderr));
+ break;
}
}
LOG("\n");
+
+ /* In benchmark mode show concise output from the main process. */
+ } else if (benchmark_output) {
+ switch (process_output_size(main_process)) {
+ case -1:
+ LOGF("%s: (unavailabe)\n", test->task_name);
+ break;
+
+ case 0:
+ LOGF("%s: (no output)\n", test->task_name);
+ break;
+
+ default:
+ LOGF("%s: ", test->task_name);
+ process_copy_output(main_process, fileno(stderr));
+ break;
+ }
}
/* Clean up all process handles. */
@@ -137,68 +165,3 @@ finalize:
return success;
}
-
-
-void log_progress(int total, int passed, int failed, char *name) {
- LOGF("[%% %3d|+ %3d|- %3d]: %-50s\n", (passed + failed) / total * 100,
- passed, failed, name);
-}
-
-
-int main(int argc, char **argv) {
- int total, passed, failed;
- test_entry_t *test;
-
-#ifdef _WIN32
- /* On windows disable the "application crashed" popup. */
- SetErrorMode(SEM_FAILCRITICALERRORS | SEM_NOGPFAULTERRORBOX |
- SEM_NOOPENFILEERRORBOX);
-#endif
-
- /* Disable stdio output buffering. */
- setvbuf(stdout, NULL, _IONBF, 0);
- setvbuf(stderr, NULL, _IONBF, 0);
-
- if (argc > 1) {
- /* A specific test process is being started. */
- for (test = (test_entry_t*)&TESTS; test->main; test++) {
- if (strcmp(argv[1], test->process_name) == 0) {
- return test->main();
- }
- }
- LOGF("Test process %s not found!\n", argv[1]);
- return 255;
-
- } else {
- /* Count the number of tests. */
- total = 0;
- test = (test_entry_t*)&TESTS;
- for (test = (test_entry_t*)&TESTS; test->main; test++) {
- if (!test->is_helper) {
- total++;
- }
- }
-
- /* Run all tests. */
- passed = 0;
- failed = 0;
- test = (test_entry_t*)&TESTS;
- for (test = (test_entry_t*)&TESTS; test->main; test++) {
- if (test->is_helper) {
- continue;
- }
-
- log_progress(total, passed, failed, test->test_name);
- rewind_cursor();
-
- if (run_test(test)) {
- passed++;
- } else {
- failed++;
- }
- }
- log_progress(total, passed, failed, "Done.");
-
- return 0;
- }
-}
diff --git a/test/test.h b/test/task.h
similarity index 93%
rename from test/test.h
rename to test/task.h
index 71db8523..bb7cbd42 100644
--- a/test/test.h
+++ b/test/task.h
@@ -19,8 +19,9 @@
* IN THE SOFTWARE.
*/
-#ifndef TEST_H_
-#define TEST_H_
+#ifndef TASK_H_
+#define TASK_H_
+
#include
#include
@@ -62,8 +63,11 @@
} while (0)
-/* Just sugar for wrapping the main() for a test. */
+/* Just sugar for wrapping the main() for a task. */
#define TEST_IMPL(name) \
- int run_##name()
+ int run_test_##name()
-#endif /* TEST_H_ */
+#define BENCHMARK_IMPL(name) \
+ int run_benchmark_##name()
+
+#endif /* TASK_H_ */
diff --git a/test/test-callback-stack.c b/test/test-callback-stack.c
index 3f3c35cc..e4377ec5 100644
--- a/test/test-callback-stack.c
+++ b/test/test-callback-stack.c
@@ -19,13 +19,15 @@
* IN THE SOFTWARE.
*/
-#include "../oio.h"
-#include "test.h"
-
-/* TODO: Add explanation of why we want on_close to be called from fresh
+/*
+ * TODO: Add explanation of why we want on_close to be called from fresh
* stack.
*/
+#include "../oio.h"
+#include "task.h"
+
+
int nested = 0;
int close_cb_called = 0;
diff --git a/test/test-fail-always.c b/test/test-fail-always.c
index 02079915..0008459e 100644
--- a/test/test-fail-always.c
+++ b/test/test-fail-always.c
@@ -19,7 +19,8 @@
* IN THE SOFTWARE.
*/
-#include "test.h"
+#include "task.h"
+
TEST_IMPL(fail_always) {
/* This test always fails. It is used to test the test runner. */
diff --git a/test/test-list.h b/test/test-list.h
index c272403d..5eda4421 100644
--- a/test/test-list.h
+++ b/test/test-list.h
@@ -26,7 +26,8 @@ TEST_DECLARE (timeout)
TEST_DECLARE (fail_always)
TEST_DECLARE (pass_always)
-TEST_LIST_START
+TASK_LIST_START
+
TEST_ENTRY (ping_pong)
TEST_HELPER (ping_pong, echo_server)
@@ -37,4 +38,4 @@ TEST_LIST_START
TEST_ENTRY (fail_always)
TEST_ENTRY (pass_always)
-TEST_LIST_END
+TASK_LIST_END
diff --git a/test/test-pass-always.c b/test/test-pass-always.c
index 3559ab2b..4fb58ff9 100644
--- a/test/test-pass-always.c
+++ b/test/test-pass-always.c
@@ -19,7 +19,8 @@
* IN THE SOFTWARE.
*/
-#include "test.h"
+#include "task.h"
+
TEST_IMPL(pass_always) {
/* This test always passes. It is used to test the test runner. */
diff --git a/test/test-ping-pong.c b/test/test-ping-pong.c
index 0c13eaf1..0c3e1dfa 100644
--- a/test/test-ping-pong.c
+++ b/test/test-ping-pong.c
@@ -20,7 +20,8 @@
*/
#include "../oio.h"
-#include "test.h"
+#include "task.h"
+
#include
#include
diff --git a/test/test-timeout.c b/test/test-timeout.c
index 94e6415c..f03bed29 100644
--- a/test/test-timeout.c
+++ b/test/test-timeout.c
@@ -20,7 +20,7 @@
*/
#include "../oio.h"
-#include "test.h"
+#include "task.h"
static int expected = 0;
diff --git a/test/runner.h b/test/runner.h
index 86c02870..a613eb24 100644
--- a/test/runner.h
+++ b/test/runner.h
@@ -19,41 +19,62 @@
* IN THE SOFTWARE.
*/
-
-#ifndef TEST_RUNNER_H_
-#define TEST_RUNNER_H_
+#ifndef RUNNER_H_
+#define RUNNER_H_
/*
- * Struct to store both tests and to define helper processes for tests.
+ * The maximum number of processes (main + helpers) that a test / benchmark
+ * can have.
+ */
+#define MAX_PROCESSES 8
+
+
+/*
+ * Struct to store both tests and to define helper processes for tasks.
*/
typedef struct {
- char *test_name;
+ char *task_name;
char *process_name;
int (*main)();
int is_helper;
-} test_entry_t;
+} task_entry_t, bench_entry_t;
+
+
+/* Runs an individual task; returns 1 if the test succeeded, 0 if it failed. */
+/* If the test fails it prints diagnostic information. */
+/* If benchmark_output is nonzero, the output from the main process is
+/* always shown. */
+int run_task(task_entry_t *test, int timeout, int benchmark_output);
/*
* Macros used by test-list.h
*/
-#define TEST_DECLARE(name) \
- int run_##name();
+#define TASK_LIST_START \
+ task_entry_t TASKS[] = {
-#define TEST_LIST_START \
- test_entry_t TESTS[] = {
-
-#define TEST_LIST_END \
+#define TASK_LIST_END \
{ 0, 0, 0, 0 } \
};
+#define TEST_DECLARE(name) \
+ int run_test_##name();
+
#define TEST_ENTRY(name) \
- { #name, #name, &run_##name, 0 },
+ { #name, #name, &run_test_##name, 0 },
#define TEST_HELPER(name, proc) \
- { #name, #proc, &run_##proc, 1 },
+ { #name, #proc, &run_test_##proc, 1 },
+#define BENCHMARK_DECLARE(name) \
+ int run_benchmark_##name();
+
+#define BENCHMARK_ENTRY(name) \
+ { #name, #name, &run_benchmark_##name, 0 },
+
+#define BENCHMARK_HELPER(name, proc) \
+ { #name, #proc, &run_benchmark_##proc, 1 },
/*
* Include platform-dependent definitions
@@ -65,12 +86,27 @@ typedef struct {
#endif
+/* The array that is filled by test-list.h or benchmark-list.h */
+extern task_entry_t TASKS[];
+
+/* Start a specific process declared by TEST_ENTRY or TEST_HELPER. */
+/* Returns the exit code of the specific process. */
+int run_task(task_entry_t *test, int timeout, int benchmark_output);
+
+/* Start a specific process declared by TEST_ENTRY or TEST_HELPER. */
+/* Returns the exit code of the specific process. */
+int run_process(char* name);
+
+
/*
* Stuff that should be implemented by test-runner-.h
* All functions return 0 on success, -1 on failure, unless specified
* otherwise.
*/
+/* Do platform-specific initialization. */
+void platform_init();
+
/* Invoke "arv[0] test-name". Store process info in *p. */
/* Make sure that all stdio output of the processes is buffered up. */
int process_start(char *name, process_info_t *p);
@@ -102,4 +138,4 @@ void process_cleanup(process_info_t *p);
/* Move the console cursor one line up and back to the first column. */
int rewind_cursor();
-#endif /* TEST_RUNNER_H_ */
+#endif /* RUNNER_H_ */
@@ -19,41 +19,62 @@
/*
- * Macros used by test-list.h
+ * Macros used by test-list.h and benchmark-list.h.
*/